! SURFACE EMISSIVITY
SUBROUTINE RSURF(F,TS,IS,S,W,X,N,RVH)
implicit none
real :: F, TS, S, W
integer :: IS, N
real :: X(*), RVH(*)
! IS --- surface type:
!        0 - Ocean
!        1 - Land
!        2 - SeaIce
!        3 - Snow
! TS --- surface temperature in K
!  F --- Frequency in GHz
!  S --- Salinity in per thousand if ocean, (negative means 35)  
!        Emissivity value, if land (negative means default,0.9)
!        Ice type, if sea ice, 0=new,1=2nd year,2=multiyear(negative:2)
!        Snow type, if snow, 0=wet,1=dry,2=refrozen (negative:1)
!  W --- Wind Speed in m/s, if ocean
!        Not used, else

if (IS.eq.0) then
  call ASSEA2(F,TS,S,W,X,N,RVH)
else if(IS.lt.4 .and. IS.gt.0) then
  call ASSEAN(F,TS,IS,S,X,N,RVH)
else
  print*,'Check, surface type:',IS
stop
end if
return
end
    
SUBROUTINE ASSEA(F,TS,S,X,N,RVH)
implicit none
real :: F,TS,S,X(*),RVH(*)
integer :: N,I
!      X ----  COS(SITA)
!      S ---- SALINTY PER MILI
COMPLEX E
     
!      Klein & Swift (1977) surface model
!      CALL DILEC2(E,F,TS,S)
!
!
!      Guillou et al. (1998) surface model
       CALL DILEC4(E,F,TS,S)
!
DO I=1,N
 CALL REF(RVH(I+N),RVH(I),E,X(I))
END DO

RETURN
END

! reference: Schluessel and Luthardt: Surface Wind Speeds Over the North Sea
!            From Special Sensor Microwave/Imager Observations,
!            JGR, 96, 4845-4853,1991.

SUBROUTINE ASSEA1(F,TS,S,W,X,N,RVH)
implicit none
REAL :: F,TS,S,W,X(*),RVH(*)
real, save :: a,b,c,d,e
DATA a,b,c,d,e/0.117,-2.09e-3,7.32e-2,0.115,3.8e-5/
real :: S0,CA,DTRV,DTRH,ROUFV,ROUFH,FV,FH,FU,ROUV,ROUH
integer :: IN,N

IF(S.lt.0.) then
 S0=35.0
ELSE
 S0=S
END IF
CALL ASSEA(F,TS,S0,X,N,RVH)
IF(W.LE.0) RETURN
       
DO IN=1,N
 CA=ACOS(X(IN))*180/3.1415926
 DTRV=W*(a+b*exp(c*CA))*SQRT(F)
 DTRH=W*(d+e*CA*CA)*SQRT(F)
 ROUV=RVH(IN)-DTRV/TS
 ROUH=RVH(IN+N)-DTRH/TS
 FV=1-9.946e-4*CA+3.218e-5*CA*CA-1.187e-6*CA*CA*CA+7e-20*CA**10
 FH=1-1.748e-3*CA-7.336e-5*CA*CA+1.044e-7*CA**3
 ROUFV=1-(208+F*1.29)/TS*FV
 ROUFH=1-(208+F*1.29)/TS*FH
 FU=2.95e-6*W**3.52
 RVH(IN+N)=ROUH*(1-FU)+ROUFH*FU
 RVH(IN)=ROUV*(1-FU)+ROUFV*FU
END DO
RETURN
END

! emissivity for non-ocean surface
!
SUBROUTINE ASSEAN(F,TS,IS,S,X,N,RVH)
implicit none
real :: F,TS,S,e
integer :: N,IS,k,i

! IS -- 1 = land, S=emissivity, 0.9 if negative given
!       2 = ice, S---0: New, 1: Second Year, 2: Multiyear
!       3 = snow,S---0: Wet, 1: Dry, 2: Refrozen

REAL :: X(*),RVH(*)
REAL,save :: CICE(4,3),CSNOW(4,3)
DATA CICE/0.95,0.95,99,0, &
          0.93,0.83,31,2, &
          0.92,0.64,31,2/
DATA CSNOW/0.76,0.99, 9,2,&
           0.90,0.75,33,3,&
           0.97,0.53,32,4/

e=0.
if (IS.eq.1) then

 e=S
 if(e.lt.0) e=0.9

else if(IS.eq.2) then

 if(S.lt.0.) then
  k=3
 else
  k=nint(S)+1
 endif
 e=(CICE(1,k)+CICE(2,k)*(F/CICE(3,k))**CICE(4,k))/ &
   (1+(F/CICE(3,k))**CICE(4,k))
        
else if(IS.eq.3) then

 if(S.lt.0.) then
  k=2
 else
  k=nint(S)+1
 endif
 e=(CSNOW(1,k)+CSNOW(2,k)*(F/CSNOW(3,k))**CSNOW(4,k))/ &
   (1+(F/CSNOW(3,k))**CSNOW(4,k))
       
end if

do i=1,N
 RVH(i+N)=1-e
 RVH(i)=1-e
end do

return
end

SUBROUTINE REF(RH,RV,EC,COSINE)
implicit none
!
!        PURPOSE
!           COMPUTE THE POWER REFLECTION COEFFICIENTS FOR A SPECULAR
!           DIELECTRIC SURFACE
!
!        DESCRIPTION OF ARGUMENTS
!           RH-CALCULATED REFLECTION COEFFICIENT FOR HORIZONTAL POLARIZA
!           RV-CALCULATED REFLECTION COEFFICIENT FOR VERTICAL POLARIZATI
!           EC-INPUT COMPLEX DIELECTRIC CONSTANT
!           COSINE-COSINE of INCIDENCE ANGLE
!
real :: RH,RV,COSINE,SINE_2
COMPLEX :: EC,Z,RV1,RH1
!     THETA = ANGLE/57.296
!     COSINE = COS(THETA)
!     Z=CSQRT(EC-SIN(THETA)**2)
SINE_2=1-COSINE**2
Z=CSQRT(EC-SINE_2)
RV1=(EC*COSINE    -Z)/(EC*COSINE    +Z)
RH1=(COSINE    -Z)/(COSINE    +Z)
RV = REAL(RV1)**2 +AIMAG(RV1)**2
RH = REAL(RH1)**2 +AIMAG(RH1)**2
RETURN
END

SUBROUTINE DILEC4(KAPPA,F,TK,S)
!   COMPUTES THE COMPLEX DIELECTRIC CONSTANT, KAPPA, FOR SEAWATER
IMPLICIT NONE
COMPLEX :: KAPPA
!   INPUT ARGUMENTS:
REAL :: F  ! FREQUENCY IN GHZ
REAL :: TK ! KELVIN TEMPERATURE
REAL :: S ! SALINITY FRACTION BY WEIGHT in Per Million
!               (FOR SEAWATER, AVERAGE S = 35)
!   11/18/98 P.Rosenkranz 
!
!   REFERENCE FOR EQUATIONS:
!   C. Guillou et al., Radio Science v.33, pp.649-667 (1998).
!
!   LOCAL VARIABLES:
REAL  :: T,R,C,EINF,ESTAT,TAU,SIGMA
!
!     S = SL*1000.
T = TK -273.15
SIGMA = .08637 + (.0306 - 4.121E-4*T)*T + &
         S*(.07745 + (.001687 + 1.937E-5*T)*T)
ESTAT = 81.82 + (-6.05E-2 + (-3.166E-2 + (3.109E-3 + &
        (-1.179E-4 + 1.483E-6*T)*T)*T)*T)*T - &
        S*(.1254 + (9.403E-3 + (-9.555E-4 +(9.088E-5 + &
        (-3.601E-6 + 4.713E-8*T)*T)*T)*T)*T)
EINF = 6.458 + (-.04203 + (-.006588 + (6.492E-4 + &
       (-1.2328E-5 + 5.043E-8*T)*T)*T)*T)*T
!  TAU IS IN PS
TAU = 17.303 + (-.6665 + (5.148E-3 + (1.2145E-3 + &
      (-5.032E-5 + 5.827E-7*T)*T)*T)*T)*T + &
      S*(-6.272E-3 + (2.357E-4 + (5.075E-4 + (-6.398E-5 + &
      (2.463E-6 - 3.066E-8*T)*T)*T)*T)*T)
R = 6.2832E-3*TAU*F
C = SIGMA/(5.5633E-2*F)
KAPPA = EINF + (ESTAT - EINF)/CMPLX(1.,R) - CMPLX(0.,C)
      
RETURN
END

SUBROUTINE DILEC2(KAPPA,F,TK,S)
implicit none
!     COMPUTES THE COMPLEX DIELECTRIC CONSTANT, KAPPA, FOR SEAWATER
!      FROM KLEIN & SWIFT, IEEE TRANS. AP-25, P.104 (1977)
!       REVISION OF STOGRYN'S EQS (IEEE TRANS. MTT-19,733,1971)
!     F = FREQUENCY IN GHZ
!     TK = KELVIN TEMPERATURE
!     S = SALINITY FRACTION BY WEIGHT (FOR SEAWATER, AVERAGE S=35)

!     I=0 FOR NACL SOLUTION
!     I=1 FOR SEAWATER
real :: F,TK,S
REAL ::  N,T,DEL,SN25,SIG,SS25,ALPHA,TAUN2,EPSN,B,A,TAU2P,EPS,R,C
COMPLEX :: KAPPA
integer :: I
!     S = SL*1000.
I=1
T = TK -273.2
DEL = 25.-T
IF(I.NE.0) GO TO 10
N = S*(1.707E-2 +1.205E-5*S + 4.058E-9*S**2)
SN25 = N*(10.394 - 2.3776*N + .68258*N**2 - .13538*N**3 + 1.0086E-2*N**4)
SIG = SN25*(1. - 1.962E-2*DEL +8.08E-5*DEL**2 -DEL*N*(3.02E-5 + &
      3.922E-5*DEL +N*(1.721E-5 -6.584E-6*DEL)))
GO TO 20
10 SS25 = S*(.182521 - 1.46192E-3*S +2.09324E-5*S**2-1.28205E-7*S**3)
ALPHA = 2.033E-2 +1.266E-4*DEL +2.464E-6*DEL**2 -S*(1.849E-5 - &
        2.551E-7*DEL +2.551E-8*DEL**2)
SIG = SS25*EXP(-DEL*ALPHA)
20 TAUN2 = 1.1109E-1 -3.824E-3*T +6.938E-5*T**2 -5.096E-7*T**3
EPSN = 87.134 -.1949*T -.01276*T**2 +2.491E-4*T**3
B = 1. +2.282E-5*S*T -7.638E-4*S -7.76E-6*S**2 +1.105E-8*S**3
A = 1. +1.613E-5*S*T -3.656E-3*S +3.21E-5*S**2 -4.232E-7*S**3
TAU2P = TAUN2*B
EPS = EPSN*A
R = TAU2P*F
C = SIG/(55.631E-3*F)
KAPPA = 4.9 + (EPS -4.9)/CMPLX(1.,R) - CMPLX(0.,C)
RETURN
END

SUBROUTINE DIFEMIS(F,TS,IS,S,WND,DIF_EMIS)
!diffused emissivity
implicit none
real :: F,TS,S,WND,DIF_EMIS
REAL :: MU(16),WTS(16),RVH(16,2)
integer :: J,IS
real EV,EH

CALL GAUSQU2 (16,MU,WTS)
CALL RSURF(F,TS,IS,S,WND,MU,16,RVH)
DIF_EMIS = 0.0
DO 200 J = 1, 16
  EV=1.0-RVH(J,1)
  EH=1.0-RVH(J,2)
  DIF_EMIS = DIF_EMIS + (EV+EH)*MU(J)*WTS(J)
200     CONTINUE

RETURN
END

SUBROUTINE FRESNEL (COSI, INDEX, EH, EV)
implicit none
COMPLEX  :: INDEX
REAL ::  COSI, EH,EV
COMPLEX  :: EPSILON, D, RH, RV

EPSILON = INDEX**2
D = CSQRT(EPSILON - 1.0 + COSI**2)
RH = (COSI - D) / (COSI + D)
RV =(EPSILON*COSI - D) / (EPSILON*COSI + D)
EH = 1.0 - CABS(RH)**2
EV = 1.0 - CABS(RV)**2

RETURN
END

SUBROUTINE GAUSQU2 (N, XA, WT)
implicit none
!  Generates the abscissas (X) and weights (W) for an N point
!  Gauss-Legendre quadrature.  
INTEGER  :: N
REAL ::  XA(*), WT(*)
INTEGER  :: K, I, J, L
REAL (kind=8) ::   X, XP, PL, PL1, PL2, DPL
REAL (kind=8), PARAMETER :: TINY=3.0D-7, PAI=3.141592654
K = (N+1)/2
DO 130 J = 1, K
 X = COS(PAI*(J-.25)/(N+.5))
 I = 0
100     CONTINUE
PL1 = 1
PL = X
DO 120 L = 2, N
 PL2 = PL1
 PL1 = PL
 PL = ( (2*L-1)*X*PL1 - (L-1)*PL2 )/L
120       CONTINUE
DPL = N*(X*PL-PL1)/(X*X-1)
XP = X
X = XP - PL/DPL
I = I+1
IF (ABS(X-XP).GT.TINY .AND. I.LT.10) GO TO 100
XA(J)     = (1-X)/2
XA(N-J+1) = (1+X)/2
WT(J)   = 1.0D0/((1.0D0-X*X)*DPL*DPL)
WT(N-J+1) = WT(J)
130   CONTINUE

RETURN
END

SUBROUTINE ASSEA2(F,TS,S,W,X,N,RVH)
! use fastem5 (from rttov11) emissivity
implicit none
integer :: N
real :: x(N),RVH(2*N),F,TS,S,W,E(4),R(4),ZEN
integer :: i,ver_rttov=5
real :: tr=1.0,rea=365.0        ! fake it

do i=1, N
ZEN=acos(x(i))*180./3.1415926
call rttov_fastem5(ver_rttov,F,ZEN,TS,S,W,E,R,tr,rea)
RVH(i)=1.0-E(1)
RVH(i+N)=1.0-E(2)
enddo

return
end
