C********************************************************************
C* Determine number concentration of cloud ice particles            *
C* Gamma distribution given by				            *
C* N=N0*D**u*exp(-l*D)						    *
C********************************************************************

        subroutine cldice(N,R0,R,NR,MODEL,IC,DEN)
	PARAMETER(SMALL=1.E-3,PAI=3.1415926,B_lamda=1.0,T_lamda=200)
        real R0(*),R(*),NR(*),DEN(*),IC,n0,mu,lamda
        character MODEL(*),cwk(6),cwk1*6
	equivalence (cwk(1),cwk1)
	real A(2), B(2), ACTsmall
	data A,B/0.0015,0.0586,0.112,0.0770/

C N --- input,number of ice particles size categories
C R0 -- input,original particle size in um (radius)
C R  -- ouput, output particle size and density. Negative values
C       The absolute value of the integer partion is particle size
C       (radius) in um, the decimal part is density in g/cm3
C NR -- output, number concentration n(r)dr in 1/m3
C MODEL - input, "TROPICS"/"tropics"
C                "MIDLAT"/"midlat" based on H2003
C         only the first letter 'M', 'm', 'T' or 't' is significant.
C       If the second letter is 'x',  the following 5 digits is FacN0
C IC - input, ice water content in g/m3
C DEN - density of individual particles
      
c	TROPICS_N0(x)=0.0015*exp(0.112*x)*FacN0
c	MidLAT_N0(x)=0.0586*exp(0.077*x)*FacN0
	U(x)=0.076*x**0.80-2
c FacN0 - Factor to N0 in standard H2003
	FacN0=1.0

        if(IC.LE.0) then
                do i=1,N
                R(i)=0
                NR(i)=0
                end do
                return
         endif

	 if(MODEL(1).eq.'M' .or. MODEL(1).eq.'m') then
		ireg=2
	 else
		ireg=1
	 endif
	
	 if(MODEL(2).eq.'x') then
           do i=1,5
	    cwk(i)=model(2+i)
	   enddo
	   cwk(6)='\0'
	   call rdf(cwk1,FacN0)
	 endif 

         do i=2,N-1
           R(i)=0.5*(R0(i+1)-R0(i-1))*2.E-4
         enddo
         R(1)=0.5*(R0(2)-R0(1))*2.E-4
         R(N)=0.5*(R0(N)-R0(N-1))*2.E-4         

c	 ncounter=0
	 ilamda=0
	 alamda=0
	 lamda=25
 8888    continue
 	 ACTsmall=SMALL
c	 ncounter=ncounter+1
c 	 print*, ncounter,lamda,alamda,ilamda,s-IC
c        if(ncounter.gt.100) then
	 if(lamda.lt.B_lamda) then
	  lamda=B_lamda
	  ACTsmall=9999999
	 else if(lamda.gt.T_lamda) then
	  lamda=T_lamda
	  ACTsmall=9999999
	 endif
c	 endif

	 mu=U(lamda)
	 n0=A(ireg)*exp(B(ireg)*lamda)*FacN0
          s=0
          do i=1,N
           DCM=R0(i)*2.E-4
           wk=n0*DCM**mu*exp(-lamda*DCM)*PAI/6*DCM**3*DEN(i)*R(i)
           s=s+wk*1.E+06
          enddo
	
	  if((s-IC).GT.ACTsmall) then
	    if(ilamda.eq.0) then
		ilamda=1
		alamda=lamda
		lamda=lamda*1.1
		goto 8888
	    else if(ilamda.gt.0) then
		alamda=lamda
		lamda=lamda*1.1
		goto 8888
	    else
		ilamda=1
		delta=lamda-alamda
		alamda=lamda
		lamda=lamda-0.5*delta
		goto 8888
	   endif
	 else if((s-IC).LT.-ACTsmall) then
	    if(ilamda.eq.0) then
		ilamda=-1
		alamda=lamda
		lamda=lamda*0.9
		goto 8888
	    else if(ilamda.lt.0) then
		alamda=lamda
		lamda=lamda*0.9
		goto 8888
	    else
		ilamda=-1
		delta=lamda-alamda
		alamda=lamda
		lamda=lamda-0.5*delta
		goto 8888
	   endif
	endif

	  if(s.gt.0) then
	  fac_residual=IC/s
	  n0=n0*fac_residual
	  endif

          do i=1,N
           DCM=R0(i)*2.E-4
           NR(i)=n0*DCM**mu*exp(-lamda*DCM)*R(i)*1.E+06
	   R(i)=-R0(i)-DEN(i)
          enddo

c	  print*,n0,lamda,mu
          return
          end
