C******************************************************
C* Determine number concentration n(r)dr and size for *
C* for cloud dropleps. Size distribution based on     *
C* Deimendjian 1964, for Stratocumulus. rc=10 um      *
C* c1=6, c2=0.5                                       *
C******************************************************

	subroutine cldwtr(N,R0,R,NR,WC)
	REAL R0(*),R(*),NR(*),WC 
C  R0 --- input, original particle size (radius) in um
C  R  --- output, particle size (radius) in um
C  N  --- input, number of particle-categories
C  NR --- output, particle number concentration n(r)dr (1/m3)
C  WC --- input, liquid water content, g/m3
C rouw = 1.0 g/cm3
        parameter(rouw=1.0)

        if(WC.LE.0) THEN
                do i=1,n
                R(i)=0
                NR(i)=0
                enddo
                return
        end if

	do i=2,N-1
	 R(i)=0.5*(R0(i+1)-R0(i-1))
	end do
	R(1)=0.5*(R0(2)-R0(1))
	R(N)=0.5*(R0(N)-R0(N-1))

	c1=6
	c2=0.5
	rc=10
	b=c1/c2/rc**c2

	s=0
	do i=1,N
	 wk=R0(I)**c1*exp(-b*R0(i)**c2)*R(i)
	 w=4./3.*3.1415926*(R0(i)*1.E-4)**3*rouw
	 s=s+w*wk
	enddo

        RN0=WC/s
	do i=1,N
	 wk=R0(i)**c1*exp(-b*R0(i)**c2)*R(i)
	 R(i)=R0(i)
	 NR(i)=RN0*wk
	enddo

	return
	end
