c main subroutine
	subroutine dmms(RR,RG,DENGRP,RS,SNWSHAPE,WR,WG,WC,WI,ICESHAPE,
     &			ICEMODEL,SNWMODEL,Pfac,dmm)
C WC: CLOUD LIQUID WATER CONTENT IN G/M3                
C WI: CLOUD ICE WATER CONTENT IN G/M3                   
C RR: RAINFALL RATE IN MM/HR,negative if use WR         
C WR: rain water content in g/m3, not used in RR>0      
C RS: SNOWFALL RATE IN MM/HR, LIQUID WATER EQUIVALENT   
C     negative if use WS
C WS: snow water content in g/m3. not used if RS>0      
C RG: Precipitation rate for Graupel, liquid equav.     
C     negative if use WG
C WG: graupel water content in g/m3. not used if RG>0 
C ICEMODEL: Cloud Ice size distribution models, choose from:
C    'TROPICS'/'tropics','MIDLAT'/'midlat',"mxNNNNN","txNNNNN" 
C ICESHAPE: particle shape number: 0 - 10 [integer]  
C SNWMODEL: Snow Particle size distribution models, choose from:
C    'SS'/'ss', 'GM'/'gm'
C SNWSHAPE: Snowflakes shape number: 0 - 10 [integer]
C DENGRP: DENSITY FOR GRAUPEL in g/cm3 (0<DENGRP<0.916),negative for default
C         value (see mwrt.inc)
C Pfac = (p0/p)**0.4  p0=surface pressure, p=pressure
C output:
c dmm -- mass median diamter, for
c        1 1    cloud liquid water drops
c        2 1    same as 1 1
c        1 2    rain drops
c        2 2    same as 1 2
c        1 3    cloud ice particles, given by maximum dimension
c        2 3    cloud ice particles, given by equal-mass solid sphere
c        1 4    snowflakes, given by maximum dimension
c        2 4    snowflakes, given by equal-mass solid sphere
c        1 5    graupels, given by maximum dimension
c        2 5    graupels, given by equal-mass solid sphere
        INCLUDE '../include/mwrt.inc'
        INTEGER ICESHAPE,SNWSHAPE
        PARAMETER(NUM_ICE=NI2-NI1+1,NUM_SNW=NS2-NS1+1,NUM_GRP=NG2-NG1+1,
     &            NUM_CLD=NC2-NC1+1,NUM_RIN=NR2-NR1+1)
	CHARACTER*10 ICEMODEL,SNWMODEL
        REAL R(NTDRP),RN(NTDRP),DEN(NTDRP)
        REAL dmm(2,5),DUM1(NTDRP*2),DUM2(NTDRP*2)
	call DRP(R,RN,RR,RG,DENGRP,WR,WG,WC,Pfac)
	F=37
	T=263.15
	IO=0
	call cspar(F,T,ICESHAPE,RZ(NI1),NUM_ICE,DUM1,DUM2,DEN(NI1),IO)
	call cspar(F,T,SNWSHAPE,RZ(NS1),NUM_SNW,DUM1,DUM2,DEN(NS1),IO)
	call PAR(R,RN,RS,WS,WI,ICEMODEL,SNWMODEL,DEN,Pfac)

	call dmmsub(R,RN,dmm)

	return
	end

C********************************************************************
C Calculate Mass Median diameter given a size distribution	    *
C********************************************************************
        subroutine dmmsub(R,RN,dmm)
	parameter (PAI=3.1415926)
	INCLUDE '../include/mwrt.inc'
        REAL R(NTDRP),RN(NTDRP)
	REAL dmm(2,5)

C R  -- input, particle size and density. Negative values for solid
C       The absolute value of the integer partion is particle size
C       (radius) in um, the decimal part is density in g/cm3
C RN -- input, number concentration n(r)dr in 1/m3
c dmm -- mass median diamter, for
c        1 1	cloud liquid water drops
c	 2 1    same as 1 1
c        1 2    rain drops
c        2 2    same as 1 2
c        1 3    cloud ice particles, given by maximum dimension
c        2 3    cloud ice particles, given by equal-mass solid sphere
c        1 4    snowflakes, given by maximum dimension
c        2 4    snowflakes, given by equal-mass solid sphere
c        1 5    graupels, given by maximum dimension
c        2 5    graupels, given by equal-mass solid sphere
c cloud water
         A=0
         do i=NC1,NC2
          DCM=R(i)*2.E-4
          A=A+PAI/6.*DCM**3*RN(i)
         enddo
         if(A.eq.0) then
          dmm(1,1)=0
          dmm(2,1)=dmm(1,1)
         else
          B=0.5*A
          A=0
          do i=NC1,NC2
          DCM=R(i)*2.E-4
          A=A+PAI/6.*DCM**3*RN(i)
          if(A.ge.B) then
           D1=R(i-1)*2
           D2=R(i)*2
           A1=A-PAI/6.*DCM**3*RN(i)
           A2=A
           goto 100
          endif
          enddo
 100      continue
          dmm(1,1)=D2-(D2-D1)/(A2-A1)*(B-A2)
          dmm(2,1)=dmm(1,1)
         endif
c rain
         A=0
         do i=NR1,NR2
          DCM=R(i)*2.E-4
          A=A+PAI/6.*DCM**3*RN(i)
         enddo
         if(A.eq.0) then
          dmm(1,2)=0
          dmm(2,2)=dmm(1,2)
         else
          B=0.5*A
          A=0
          do i=NR1,NR2
          DCM=R(i)*2.E-4
          A=A+PAI/6.*DCM**3*RN(i)
          if(A.ge.B) then
           D1=R(i-1)*2
           D2=R(i)*2
           A1=A-PAI/6.*DCM**3*RN(i)
           A2=A
           goto 200
          endif
          enddo
 200      continue
          dmm(1,2)=D2-(D2-D1)/(A2-A1)*(B-A2)
          dmm(2,2)=dmm(1,2)
         endif
c cloud ice
         A=0
         do i=NI1,NI2
          DCM=int(-R(i))*2.E-4
          DEN=-R(i)-int(-R(i))
          A=A+PAI/6.*DCM**3*RN(i)*DEN
         enddo
         if(A.eq.0) then
          dmm(1,3)=0
          dmm(2,3)=dmm(1,3)
         else
          B=0.5*A
          A=0
          do i=NI1,NI2
          DCM=int(-R(i))*2.E-4
          DEN=-R(i)-int(-R(i))
          A=A+PAI/6.*DCM**3*RN(i)*DEN
          if(A.ge.B) then
           D1=R(i-1)*2
           DEN1=-R(i-1)-int(-R(i-1))
           D2=R(i)*2
           E1=D1*(DEN1/DPI)**(1./3.)
           E2=D2*(DEN/DPI)**(1./3.)
           A1=A-PAI/6.*DCM**3*RN(i)*DEN
           A2=A
           goto 300
          endif
          enddo
 300      continue
          dmm(1,3)=D2-(D2-D1)/(A2-A1)*(B-A2)
          dmm(2,3)=E2-(E2-E1)/(A2-A1)*(B-A2)
         endif
c snow
         A=0
         do i=NS1,NS2
          DCM=int(-R(i))*2.E-4
          DEN=-R(i)-int(-R(i))
          A=A+PAI/6.*DCM**3*RN(i)*DEN
         enddo
         if(A.eq.0) then
          dmm(1,4)=0
          dmm(2,4)=dmm(1,4)
         else
          B=0.5*A
          A=0
          do i=NS1,NS2
          DCM=int(-R(i))*2.E-4
          DEN=-R(i)-int(-R(i))
          A=A+PAI/6.*DCM**3*RN(i)*DEN
          if(A.ge.B) then
           D1=R(i-1)*2
           DEN1=-R(i-1)-int(-R(i-1))
           D2=R(i)*2
           E1=D1*(DEN1/DPI)**(1./3.)
           E2=D2*(DEN/DPI)**(1./3.)
           A1=A-PAI/6.*DCM**3*RN(i)*DEN
           A2=A
           goto 400
          endif
          enddo
 400      continue
          dmm(1,4)=D2-(D2-D1)/(A2-A1)*(B-A2)
          dmm(2,4)=E2-(E2-E1)/(A2-A1)*(B-A2)
         endif
c graupel
         A=0
         do i=NG1,NG2
          DCM=int(-R(i))*2.E-4
          DEN=-R(i)-int(-R(i))
          A=A+PAI/6.*DCM**3*RN(i)*DEN
         enddo
         if(A.eq.0) then
          dmm(1,5)=0
          dmm(2,5)=dmm(1,5)
         else
          B=0.5*A
          A=0
          do i=NG1,NG2
          DCM=int(-R(i))*2.E-4
          DEN=-R(i)-int(-R(i))
          A=A+PAI/6.*DCM**3*RN(i)*DEN
          if(A.ge.B) then
           D1=R(i-1)*2
           DEN1=-R(i-1)-int(-R(i-1))
           D2=R(i)*2
           E1=D1*(DEN1/DPI)**(1./3.)
           E2=D2*(DEN/DPI)**(1./3.)
           A1=A-PAI/6.*DCM**3*RN(i)*DEN
           A2=A
           goto 500
          endif
          enddo
 500      continue
          dmm(1,5)=D2-(D2-D1)/(A2-A1)*(B-A2)
          dmm(2,5)=E2-(E2-E1)/(A2-A1)*(B-A2)
         endif

          return
          end
