C*********************************************************************
C*  Determine number concentration for graupels                      *
C*  Size distribution takes the form n(r)=n0*exp(-gamma*r)           *
C*  Depending on n0 and gamma, we use distributions of               *
C*  M-P                                                              *
C*********************************************************************
      subroutine graupel(N,R0,R,NR,Rou,GR,GC,DPI,Pfac)
      real R0(*),R(*),NR(*),Rou,GR,GC,Pfac,DPI
C N --- Input, number of graupel size categories
C R0 -- input, size of graupel in um(radius), maximum dimension
C R --- output, negative values. The absolute value of integer part is
C       the size in um of particle (radius), and the decimal
C       portion is density in g/cm3 of the particle
C NR --- output, number concentration of n(r)dr in 1/m3
C Rou -- input, density of graupels. If Rou<=0, we will use the DPI
C GR -- input, snowfall rate in mm/h, use GC if GR is negative
C GC -- input, snow water content in g/m3, use GR is GC is negative
C    IF BOTH GR and GC are positive, GR is used and GC is ignored
C DPI -- input, Density for solid ice in g/cm3, (0.916)
C Pfac -- input, (p0/p)^0.4 

      IF(GR.EQ.0) THEN
                do i=1,N
                R(i)=0
                NR(i)=0
                enddo
                return
      END IF

      IF(GR.LE.0 .and. GC.LE.0) THEN
                do i=1,N
                R(i)=0
                NR(i)=0
                enddo
                return
      END IF

      do i=2, N-1
         R(i)=0.5*(R0(i+1)-R0(i-1))*2
      enddo
      R(1)=0.5*(R0(2)-R0(1))*2
      R(N)=0.5*(R0(N)-R0(N-1))*2

      if(Rou.gt.DPI .or. Rou.le.0) Rou=DPI

C SIZE DISTRIBUTIONS
C M-P
        if(GR.GT.0) then
                gamma=41*GR**(-0.21)*2*1.E-4
        else
                N0S=8
                gamma=(3.1415926*Rou*N0S/GC)**0.25*1.E-3*2
        endif

         IF(GR.GT.0) THEN
           s=0
           do i=1,N
            wk=exp(-gamma*R0(i))
            s=s+wk*VG(R0(i),Pfac)*VOLG(R0(i))*R(i)*Rou
           enddo
           RN0=GR/s/3.6
           do i=1,N
            wk=RN0*exp(-gamma*R0(i))
            NR(i)=wk*R(i)
            R(i)=-R0(i)-Rou
           enddo
         ELSE 
           s=0
           do i=1,N
            wk=exp(-gamma*R0(i))
            s=s+wk*VOLG(R0(i))*R(i)*Rou
           enddo
           RN0=GC/s
           do i=1,N
            wk=RN0*exp(-gamma*R0(i))
            NR(i)=wk*R(i)
            R(i)=-R0(i)-Rou
           enddo
          END IF
       
          return
          end

C Terminal Velocity in m/s
C Locatelli and Hobbs (1974)  (Houze book p.93)
        FUNCTION VG(R,Pfac)
         dr=2*R*1.E-3
         VG=1.2*dr**0.65*Pfac
        return
        end

C Volume in cm3
        FUNCTION VOLG(R)
         dr=2*R*1.E-4
         VOLG=3.1415926/6*dr*dr*dr
        return
        end
