C*****************************************************************
C Determine rain drop concentration given                        *
C  rainfall rate or rain water content using M-P distribution    *
C Apply (p0/p)^0.4 for fallspeed change with pressure            *
C see Foote and DuToit (1969)                                    *
C Refer Rutledge and Hobbs (1983) for parameters used here       *
C*****************************************************************
      subroutine rain(N,R0,R,NR,RR,RC,Pfac)
        real R0(*),R(*),NR(*),RR,RC,Pfac
C N ---- input, number of raindrop categories
C R0 --- input, original particle size (radius) in um
C R ---- output, particle size in um
C NR --- output, number concentration n(r)dr in 1/m3
C RR --- input, rainfall rate in mm/h, if RR<0, RC must be given
C RC --- input, rain water content, in g/m3, if RC<0, RR must be given
C if both RC and RR are positive, RR is used and RC is ignored.
C Pfac --- input,  (p0/p)^0.4
C rouw = 1.0 g/cm3
        parameter(rouw=1.0)
       IF(RR.EQ.0) THEN
                do i=1,N
                R(i)=0
                NR(i)=0
                enddo
                return
        END IF

        IF(RR.LE.0 .and. RC.LE.0) THEN
                do i=1,N
                R(i)=0
                NR(i)=0
                enddo
                return
        END IF

        do i=2, N-1
         R(i)=0.5*(R0(i+1)-R0(i-1))*2
        enddo
        R(1)=0.5*(R0(2)-R0(1))*2
        R(N)=0.5*(R0(N)-R0(N-1))*2

        IF(RR.GT.0) THEN
           gamma=8.2*RR**(-0.21)*1.E-3
           s=0
           do i=1,N
            wk=exp(-gamma*R0(i))
            s=s+wk*VR(R0(i),Pfac)*VOLR(R0(i))*R(i)
           enddo
c VOLR in cm3 (3.6E6 * 1.E-6 = 3.6)
c (s: m/s*cm3 ==> mm/hr*m3)
           RN0=RR/s/3.6
           do i=1,N
            wk=RN0*exp(-gamma*R0(i))
            NR(i)=wk*R(i)
            R(i)=R0(i)
           enddo
         ELSE 
           gamma=(8*3.1415926/RC)**0.25*1.E-3*2
           s=0
           do i=1,N
           wk=exp(-gamma*R0(i))
           s=s+wk*VOLR(R0(i))*rouw*R(i)
           enddo
c VOLR(cm3)*rouw(g/cm3) ==> g
c RC(g/m3) / s(g) ==> 1/m3
           RN0=RC/s
           do i=1,N
            wk=RN0*exp(-gamma*R0(i))
            NR(i)=wk*R(i)
            R(i)=R0(i)
           enddo
          END IF

          return
          end

C Terminal Velocity in m/s
        FUNCTION VR(R,Pfac)
         dr=2*R*1.E-4
         VR=(-0.267+51.5*dr-102.25*dr*dr+75.5*dr*dr*dr)*Pfac
        return
        end

C Volume in cm3
        FUNCTION VOLR(R)
         dr=2*R*1.E-4
         VOLR=3.1415926/6*dr*dr*dr
        return
        end

