C*********************************************************************
C*  Determine number concentration for snow particles                *
C*  Size distribution takes the form n(r)=n0*exp(-gamma*r)           *
C*  Depending on n0 and gamma, we use distributions of               *
C*  S-S, G-M,                                                        *
C*********************************************************************
C
      subroutine snow(N,R0,R,NR,SR,SC,den,SNWMODEL,Pfac)

      real R0(*),R(*),NR(*),SR,SC,Pfac,den(*)
      character SNWMODEL

C N --- Input, number of snow particle categories
C R0 -- input, size of snow particles in um, 1/2 of maximum dimension
C R --- output, negative values. The absolute value of integer part is
C       the size in um of particle (1/2 maximum dimension), and the decimal
C       portion is density in g/cm3 of the particle
C NR --- output, number concentration of n(r)dr in 1/m3
C SR -- input, snowfall rate in mm/h, use SC if SR is negative
C SC -- input, snow water content in g/m3, use SR is SC is negative
C    IF BOTH SR and SC are positive, SR is used and SC is ignored
C Pfac -- input, (p0/p)^0.4 
C den - input, workspace

      IF(SR.EQ.0) THEN
                do i=1,N
                R(i)=0
                NR(i)=0
                enddo
                return
      END IF

      IF(SR.LE.0 .and. SC.LE.0) THEN
                do i=1,N
                R(i)=0
                NR(i)=0
                enddo
                return
      END IF

      do i=2, N-1
         R(i)=0.5*(R0(i+1)-R0(i-1))*2
      enddo
      R(1)=0.5*(R0(2)-R0(1))*2
      R(N)=0.5*(R0(N)-R0(N-1))*2
      
      IF(SNWMODEL.NE."G" .and. SNWMODEL.NE."g") GOTO 1111

C SIZE DISTRIBUTIONS
C Gunn and Marshall (1958)
        if(SR.GT.0) then
                gamma=25.5*SR**(-0.48)*2*1.E-4
        else
c               Rou0=0.1
c               N0S=20            !see R&H (1983)
                Rou0=0.2
                N0S=8
                gamma=(3.1415926*Rou0*N0S/SC)**0.25*1.E-3*2
        endif

         IF(SR.GT.0) THEN
           s=0
           do i=1,N
            EQD=R0(i)
            wk=exp(-gamma*EQD)
            s=s+wk*VS(R0(i),Pfac)*VOLS(R0(i))*den(i)*R(i)
           enddo
           RN0=SR/s/3.6
           do i=1,N
            EQD=R0(i)
            wk=RN0*exp(-gamma*EQD)
            NR(i)=wk*R(i)
            R(i)=-R0(i)-den(i)
           enddo
         ELSE
           s=0
           do i=1,N
            EQD=R0(i)
            wk=exp(-gamma*EQD)
            s=s+wk*VOLS(R0(i))*den(i)*R(i)
           enddo
           RN0=SC/s
           do i=1,N
            EQD=R0(i)
            wk=RN0*exp(-gamma*EQD)
            NR(i)=wk*R(i)
            R(i)=-R0(i)-den(i)
           enddo
          END IF
                                                                                         
	  return

 1111   CONTINUE

C Sekhon & Srivasta (1970)
        if(SR.GT.0) then
                gamma=22.9*SR**(-0.45)*2*1.E-4
        else
C               Rou0=0.1
C               N0S=20            !see R&H (1983)
                Rou0=0.2
                N0S=8
                gamma=(3.1415926*Rou0*N0S/SC)**0.25*1.E-3*2
        endif

         IF(SR.GT.0) THEN
           s=0
           do i=1,N
            DenFac=den(i)**(1./3.)
            EQD=R0(i)*Denfac
            wk=exp(-gamma*EQD)
            s=s+wk*VS(R0(i),Pfac)*VOLS(R0(i))*den(i)*R(i)*DenFac
           enddo
           RN0=SR/s/3.6
           do i=1,N
            DenFac=den(i)**(1./3.)
            EQD=R0(i)*DenFac
            wk=RN0*exp(-gamma*EQD)
            NR(i)=wk*R(i)*DenFac
            R(i)=-R0(i)-den(i)
           enddo
         ELSE 
           s=0
           do i=1,N
            DenFac=den(i)**(1./3.)
            EQD=R0(i)*DenFac
            wk=exp(-gamma*EQD)
            s=s+wk*VOLS(R0(i))*den(i)*R(i)*DenFac
           enddo
           RN0=SC/s
           do i=1,N
            DenFac=den(i)**(1./3.)
            EQD=R0(i)*DenFac
            wk=RN0*exp(-gamma*EQD)
            NR(i)=wk*R(i)*DenFac
            R(i)=-R0(i)-den(i)
           enddo
          END IF

          return
          end

C Terminal Velocity in m/s
C Rutledge and Hobbs (1983)
        FUNCTION VS(R,Pfac)
         dr=2*R*1.E-6
         VS=1.139*dr**0.11*Pfac
        return
        end

C Volume in cm3
        FUNCTION VOLS(R)
         dr=2*R*1.E-4
         VOLS=3.1415926/6*dr*dr*dr
        return
        end
