C SUBROUTINES FOR CALCULATIONS OF SURFACE EMISSIVITY
       SUBROUTINE RSURF(F,TS,IS,S,W,X,N,RVH)
C IS --- surface type:
C        0 - Ocean
C        1 - Land
C        2 - SeaIce
C        3 - Snow
C TS --- surface temperature in K
C  F --- Frequency in GHz
C  S --- Salinity in per thousand if ocean, (negative means 35)  
C        Emissivity value, if land (negative means default,0.9)
C        Ice type, if sea ice, 0=new,1=2nd year,2=multiyear(negative:2)
C        Snow type, if snow, 0=wet,1=dry,2=refrozen (negative:1)
C  W --- Wind Speed in m/s, if ocean
C        Not used, else

       REAL X(*),RVH(*)
       if (IS.eq.0) then
         call ASSEA1(F,TS,S,W,X,N,RVH)
       else if(IS.lt.4 .and. IS.gt.0) then
         call ASSEAN(F,TS,IS,S,X,N,RVH)
       else
         print*,'Check, surface type:',IS
         stop
       end if
       return
       end
    
       SUBROUTINE ASSEA(F,TS,S,X,N,RVH)
C      X ----  COS(SITA)
C      S ---- SALINTY PER MILI
       REAL X(*),RVH(*)
       COMPLEX E
c     
c      Klein & Swift (1977) surface model
c      CALL DILEC2(E,F,TS,S)
c
c
c      Guillou et al. (1998) surface model
       CALL DILEC4(E,F,TS,S)
C
       DO I=1,N
	CALL REF(RVH(I+N),RVH(I),E,X(I))
       END DO

       RETURN
       END
CCF

c reference: Schluessel and Luthardt: Surface Wind Speeds Over the North Sea
c            From Special Sensor Microwave/Imager Observations,
c            JGR, 96, 4845-4853,1991.

       SUBROUTINE ASSEA1(F,TS,S,W,X,N,RVH)
       REAL X(*),RVH(*)
       SAVE a,b,c,d,e
       DATA a,b,c,d,e/0.117,-2.09e-3,7.32e-2,0.115,
     1                3.8e-5/

       IF(S.lt.0.) then
 	S0=35.0
       ELSE
	S0=S
       END IF
       CALL ASSEA(F,TS,S0,X,N,RVH)
       IF(W.LE.0) RETURN
       
       DO IN=1,N
        CA=ACOS(X(IN))*180/3.1415926
        DTRV=W*(a+b*exp(c*CA))*SQRT(F)
        DTRH=W*(d+e*CA*CA)*SQRT(F)
        ROUV=RVH(IN)-DTRV/TS
        ROUH=RVH(IN+N)-DTRH/TS
        FV=1-9.946e-4*CA+3.218e-5*CA*CA
     1      -1.187e-6*CA*CA*CA+7e-20*CA**10
        FH=1-1.748e-3*CA-7.336e-5*CA*CA+1.044e-7*CA**3
        ROUFV=1-(208+F*1.29)/TS*FV
        ROUFH=1-(208+F*1.29)/TS*FH
        FU=2.95e-6*W**3.52
        RVH(IN+N)=ROUH*(1-FU)+ROUFH*FU
        RVH(IN)=ROUV*(1-FU)+ROUFV*FU
       END DO
       RETURN
       END

C emissivity for non-ocean surface
C
       SUBROUTINE ASSEAN(F,TS,IS,S,X,N,RVH)

C IS -- 1 = land, S=emissivity, 0.9 if negative given
C       2 = ice, S---0: New, 1: Second Year, 2: Multiyear
C       3 = snow,S---0: Wet, 1: Dry, 2: Refrozen

       REAL X(*),RVH(*),CICE(4,3),CSNOW(4,3)
       save CICE,CSNOW
       DATA CICE/0.95,0.95,99,0,
     1           0.93,0.83,31,2,
     2           0.92,0.64,31,2/
       DATA CSNOW/0.76,0.99, 9,2,
     1            0.90,0.75,33,3,
     2            0.97,0.53,32,4/

       e=0.
       if (IS.eq.1) then
         e=S
         if(e.lt.0) e=0.9
       else if(IS.eq.2) then
         if(S.lt.0.) then
	 k=3
	 else
         k=nint(S)+1
	 endif
         e=(CICE(1,k)+CICE(2,k)*(F/CICE(3,k))**CICE(4,k))
     1     /(1+(F/CICE(3,k))**CICE(4,k))
        
       else if(IS.eq.3) then
         if(S.lt.0.) then
	 k=2
	 else
         k=nint(S)+1
	 endif
         e=(CSNOW(1,k)+CSNOW(2,k)*(F/CSNOW(3,k))**CSNOW(4,k))
     1     /(1+(F/CSNOW(3,k))**CSNOW(4,k))
       end if

       do i=1,N
        RVH(i+N)=1-e
        RVH(i)=1-e
       end do

       return
       end


      SUBROUTINE REF(RH,RV,EC,COSINE)
C
C        PURPOSE
C           COMPUTE THE POWER REFLECTION COEFFICIENTS FOR A SPECULAR
C           DIELECTRIC SURFACE
C
C        DESCRIPTION OF ARGUMENTS
C           RH-CALCULATED REFLECTION COEFFICIENT FOR HORIZONTAL POLARIZA
C           RV-CALCULATED REFLECTION COEFFICIENT FOR VERTICAL POLARIZATI
C           EC-INPUT COMPLEX DIELECTRIC CONSTANT
C           COSINE-COSINE of INCIDENCE ANGLE
C
      COMPLEX EC,Z,RV1,RH1
c     THETA = ANGLE/57.296
c     COSINE = COS(THETA)
c     Z=CSQRT(EC-SIN(THETA)**2)
      SINE_2=1-COSINE**2
      Z=CSQRT(EC-SINE_2)
      RV1=(EC*COSINE    -Z)/(EC*COSINE    +Z)
      RH1=(COSINE    -Z)/(COSINE    +Z)
      RV = REAL(RV1)**2 +AIMAG(RV1)**2
      RH = REAL(RH1)**2 +AIMAG(RH1)**2
      RETURN
      END

      SUBROUTINE DILEC4(KAPPA,F,TK,S)
C   COMPUTES THE COMPLEX DIELECTRIC CONSTANT, KAPPA, FOR SEAWATER
      IMPLICIT NONE
      COMPLEX KAPPA
C   INPUT ARGUMENTS:
      REAL F  ! FREQUENCY IN GHZ
      REAL TK ! KELVIN TEMPERATURE
      REAL S ! SALINITY FRACTION BY WEIGHT in Per Million
C               (FOR SEAWATER, AVERAGE S = 35)
C   11/18/98 P.Rosenkranz 
c
C   REFERENCE FOR EQUATIONS:
c   C. Guillou et al., Radio Science v.33, pp.649-667 (1998).
c
C   LOCAL VARIABLES:
      REAL  T,R,C,EINF,ESTAT,TAU,SIGMA
C
C     S = SL*1000.
      T = TK -273.15
      SIGMA = .08637 + (.0306 - 4.121E-4*T)*T
     & + S*(.07745 + (.001687 + 1.937E-5*T)*T)
      ESTAT = 81.82 + (-6.05E-2 + (-3.166E-2 + (3.109E-3 
     & + (-1.179E-4 + 1.483E-6*T)*T)*T)*T)*T 
     & - S*(.1254 + (9.403E-3 + (-9.555E-4 +(9.088E-5
     & + (-3.601E-6 + 4.713E-8*T)*T)*T)*T)*T)
      EINF = 6.458 + (-.04203 + (-.006588 + (6.492E-4
     & + (-1.2328E-5 + 5.043E-8*T)*T)*T)*T)*T
C  TAU IS IN PS
      TAU = 17.303 + (-.6665 + (5.148E-3 + (1.2145E-3
     & + (-5.032E-5 + 5.827E-7*T)*T)*T)*T)*T
     & + S*(-6.272E-3 + (2.357E-4 + (5.075E-4 + (-6.398E-5
     & + (2.463E-6 - 3.066E-8*T)*T)*T)*T)*T)
      R = 6.2832E-3*TAU*F
      C = SIGMA/(5.5633E-2*F)
      KAPPA = EINF + (ESTAT - EINF)/CMPLX(1.,R) - CMPLX(0.,C)
      RETURN
      END

      SUBROUTINE DILEC2(KAPPA,F,TK,S)
C     COMPUTES THE COMPLEX DIELECTRIC CONSTANT, KAPPA, FOR SEAWATER
C       FROM KLEIN & SWIFT, IEEE TRANS. AP-25, P.104 (1977)
C       REVISION OF STOGRYN'S EQS (IEEE TRANS. MTT-19,733,1971)
C     F = FREQUENCY IN GHZ
C     TK = KELVIN TEMPERATURE
C     S = SALINITY FRACTION BY WEIGHT (FOR SEAWATER, AVERAGE S=35)

C     I=0 FOR NACL SOLUTION
C     I=1 FOR SEAWATER

      REAL   N
      COMPLEX  KAPPA
c     S = SL*1000.
      I=1
      T = TK -273.2
      DEL = 25.-T
      IF(I.NE.0) GO TO 10
      N = S*(1.707E-2 +1.205E-5*S + 4.058E-9*S**2)
      SN25 = N*(10.394 - 2.3776*N + .68258*N**2 - .13538*N**3 +
     1 1.0086E-2*N**4)
      SIG = SN25*(1. - 1.962E-2*DEL +8.08E-5*DEL**2 -DEL*N*(3.02E-5 +
     1 3.922E-5*DEL +N*(1.721E-5 -6.584E-6*DEL)))
      GO TO 20
   10 SS25 = S*(.182521 - 1.46192E-3*S +2.09324E-5*S**2-1.28205E-7*S**3)
      ALPHA = 2.033E-2 +1.266E-4*DEL +2.464E-6*DEL**2 -S*(1.849E-5
     1 -2.551E-7*DEL +2.551E-8*DEL**2)
      SIG = SS25*EXP(-DEL*ALPHA)
   20 TAUN2 = 1.1109E-1 -3.824E-3*T +6.938E-5*T**2 -5.096E-7*T**3
      EPSN = 87.134 -.1949*T -.01276*T**2 +2.491E-4*T**3
      B = 1. +2.282E-5*S*T -7.638E-4*S -7.76E-6*S**2 +1.105E-8*S**3
      A = 1. +1.613E-5*S*T -3.656E-3*S +3.21E-5*S**2 -4.232E-7*S**3
      TAU2P = TAUN2*B
      EPS = EPSN*A
      R = TAU2P*F
      C = SIG/(55.631E-3*F)
      KAPPA = 4.9 + (EPS -4.9)/CMPLX(1.,R) - CMPLX(0.,C)
      RETURN
      END
