C MAIN SUBROUTINE PROGRAM FOR  4-stream h-g program
C VERSION 2006.5.21

       SUBROUTINE MWSTRM4(F,L,LOUT,IHV,U,WC,WI,RR,WR,RS,WS,RG,WG,PMB,TK,
     1  RH,Z,TS,IS,SLNTY,WND,IceModel,IceShape,SnwModel,SnwShape,DENGRP,
     2  MixingRule,TB,TW1,TW2,WK,SurfModel,IO)

C
C ********************** INPUT ***************************
C
C F: Frequency in GHz
C L: TOTAL NUMBER OF LAYERS
C LOUT: Layer Number (top=1, bottom=L) at which TB will be outputed
C IHV: 1=V-POL, 2=H-POL, else=BOTH
C U: DIRECTION for OUTPUT, positive upward TB at layer top
C                       negative downward TB at layer bottom
C -------
C VERTICAL PROFILES: FIRST LAYER IS TOP, DOWNWARD
C                    DIMENSION = L / L+1
C
C  Variables with dimension of L are average in a layer
C                              L+1 are value at the top boundary
C  For example, WC(1) is the LWC averaged in the first (top) layer
C   while TK(1) is the Temp at TOA, TK(L+1) the air Temp at surface 
C WC: CLOUD LIQUID WATER CONTENT IN G/M3		WC(L)
C WI: CLOUD ICE WATER CONTENT IN G/M3			WI(L)
C RR: RAINFALL RATE IN MM/HR,negative if use WR		RR(L)
C WR: rain water content in g/m3, not used in RR>0      WR(L)
C RS: SNOWFALL RATE IN MM/HR, LIQUID WATER EQUIVALENT	RS(L)
C     negative if use WS
C WS: snow water content in g/m3. not used if RS>0      WS(L)
C RG: Precipitation rate for Graupel, liquid equav.     RG(L)
C     negative if use WG
C WG: graupel water content in g/m3. not used if RG>0   WG(L)
C PMB: PRESSURE IN mb					PMB(L+1)
C TK: TEMPERATURE IN K					TK(L+1)
C RH: RELATIVE HUMIDITY IN %				RH(L+1)
C Z: LAYER TOP ALTITUDE IN m				Z(L+1)
C -------
C TS: SURFACE TEMPERATURE IN K
C IS: SURFACE TYPE:
C       0 = OCEAN
C       1 = LAND or SPECIFIED SURFACE EMISSIVITY
C       2 = SEA ICE
C       3 = SNOW
C SLNTY: (negative means using default)
C       WHEN IS=0, SALINITY IN PER THOUSAND [35]
C       WHEN IS=1, EMISSIVITY [0.9]
C       WHEN IS=2, ICE TYPE:0=NEW,1=2ND YEAR, 2=MULTIYEAR [2]
C       WHEN IS=3, SNOW TYPE: 0=WET, 1=DRY, 2=REFROZEN [1]
C WND: WIND SPEED (ONLY USED WHEN IS=0)
C ICEMODEL: Cloud Ice size distribution models, choose from:
C    'TROPICS'/'tropics','MIDLAT'/'midlat' 
C ICESHAPE: particle shape number: 0 - 10 [integer]  
C SNWMODEL: Snow Particle size distribution models, choose from:
C    'SS'/'ss', 'GM'/'gm'
C SNWSHAPE: Snowflakes shape number: 0 - 10 [integer]
C DENGRP: DENSITY FOR GRAUPEL in g/cm3 (0<DENGRP<0.916),negative for default
C         value (see mwrt.inc)
C MixingRule: Mixing Rule for graupel: MG,LL/DB,EV,EM/BR
C
C SurfModel: "Fresnel" [default], or "Lambertian"
C            character*1, Valid values are "l","L","f","F"
C
C IO: set to 0 when call this program first time, and never change it
C------------------------------------------------------------------ 
C
C  ********************* OUTPUT **************************
C
C TB(2): BRIGHTNESS TEMPERATURE (K)
C 1= Vertical Pol
C 2= Horizontal Pol
C If IHV Specify only one pol, The other will be blank. 
C TW1: TOTAL INTEGRATED LIQUID WATER (CLOUD AND RAIN)
C TW2: TOTAL INTEGRATED ICE WATER (CLOUD ICE AND SNOW)
C
C---------------------------------------------------------
C
C ******** THE FOLLOWING VARIABLE IS WORKING SPACE **********
C
C    WK(95*L)
C
C -------------------------------------------------------------
C
	DIMENSION WC(*),WI(*),RR(*),RS(*),PMB(*),TK(*),RH(*),Z(*),
     1            TB(*),WK(*),RG(*),WR(*),WS(*),WG(*)
	CHARACTER*(*) IceModel,SurfModel,SnwModel,MixingRule
	INTEGER IceShape,SnwShape

	CALL RAD(F,L,LOUT,IHV,U,WC,WI,RR,WR,RS,WS,RG,WG,PMB,TK,RH,Z,
     1   TS,IS,SLNTY,WND,IceModel,IceShape,SnwModel,SnwShape,DENGRP,
     a   MixingRule,TB, TW1,TW2,WK,
     2   WK(64*L+1),WK(68*L+1),WK(70*L+1),WK(78*L+1),WK(79*L+1),
     3   WK(83*L+1),WK(87*L+1),WK(88*L+1),WK(93*L+1),WK(94*L+1),
     4   SurfModel,IO)

	RETURN
	END


C ====================
C This is the part that does real work
C

        INCLUDE '../include/4strm.inc'

        SUBROUTINE RAD(F,L,LOUT,IHV,U,WC,WI,RR,WR,RS,WS,RG,WG,PMB,TK,RH,
     1    Z,TS,IS,SLNTY,WND,ICEMODEL,ICESHAPE,SNWMODEL,SNWSHAPE,DENGRP,
     2    MixingRule,TB,TW1,TW2,AX,BX,EKJ,GIJ,TAU,WK,PJ,OM,AL0,B0,B1,
     3    SurfModel,IO)

	COMMON /strm4/AJ(2),UI(2)

	include '../include/mwrt.inc'

	DIMENSION WC(*),WI(*),RR(*),RS(*),PMB(*),TK(*),RH(*),Z(*),
     &   EKJ(2,*),GIJ(4,2,*),AX(*),BX(*),TB(*),RSUU(2),TAU(*),
     &   WK(*),OM(*),B0(*),B1(*),PJ(4,*),AL0(0:4,*),RG(*),WR(*),WS(*),
     &   WG(*)
	CHARACTER*(*) ICEMODEL,SurfModel,SNWMODEL,MixingRule
	PARAMETER (NR=NTDRP,NAB=MAXNAB,NS=2,NL=2*NS-1)
	PARAMETER (REAL_SMALL=1.0E-5,OM_SMALL=0.01,ALMOST1=0.99999)
	DIMENSION R(NR),NABR(NR),RN(NR),RC(3),TB0(2),AL(0:NL+1),RCWORK(3)
	DIMENSION CS(2,NR),GN(NR),DEN(NR) 
	COMPLEX AN(NR*NAB),BN(NR*NAB)
	INTEGER ICESHAPE,SNWSHAPE
	PARAMETER(NUM_ICE=NI2-NI1+1,NUM_SNW=NS2-NS1+1,NUM_GRP=NG2-NG1+1,
     &            NUM_CLD=NC2-NC1+1,NUM_RIN=NR2-NR1+1)

c2003.12.3 LIU - use V-H-averaged emissivity for diffused intensity
c                calculation. So far, considered H and V independent
c       DIMENSION   RSV(NS),RSH(NS), RSU(2*NS)
c	EQUIVALENCE (RSU(1),RSV(1)),(RSU(3),RSH(1))
	DIMENSION RSU_DIF(NS) 

	IF (LOUT.GT.L .or. LOUT.LT.1) THEN
	 PRINT*,"LOUT MUST BE BETWEEN 1 AND ",L
	 PRINT*,"CURRENT LOUT=",LOUT
	ENDIF
	IF (SurfModel.eq.'') SurfModel = 'F'

	TW1=0.0
	TW2=0.0

c2003.12.3 Liu
c	CALL RSURF(F,TS,IS,SLNTY,WND,UI,NS,RSU)
        call DIFEMIS(F,TS,IS,SLNTY,WND,DIF_EMIS)
	DO I=1,NS
	 RSU_DIF(I)=1.-DIF_EMIS
	ENDDO
c
	CALL RSURF(F,TS,IS,SLNTY,WND,U,1,RSUU)

	OM_MAX=0.0

	DO I=1,L

	 DO J=1,NR
           GN(J)=0.0
           CS(1,J)=0.0
           CS(2,J)=0.0
         ENDDO

	 AVET=(TK(I)+TK(I+1))*0.5
	 AVEP=(PMB(I)+PMB(I+1))*0.5
	 AVERH=(RH(I)+RH(I+1))*0.5
	 DZ=Z(I)-Z(I+1)
	 CALL ABSORB(0,AVET,AVEP,AVERH,F,AIRABS)
         Pfac=(PMB(L+1)/AVEP)**0.4
c        print*,Pfac,WC(I),WI(I),WR(I),WS(I),WG(I),RR(I),RS(I),RG(I)

	 CALL DRP(R,RN,RR(I),RG(I),DENGRP,WR(I),WG(I),WC(I),Pfac)
c Cloud liquid water
	 call ABSUB(F,AVET,R(NC1),AN,BN,NUM_CLD,NAB,NABR,MixingRule)
	 call csdrp(F,NUM_CLD,CS(1,NC1),GN(NC1),AN,BN,NABR)
c Rain 
         call ABSUB(F,AVET,R(NR1),AN,BN,NUM_RIN,NAB,NABR,MixingRule)
         call csdrp(F,NUM_RIN,CS(1,NR1),GN(NR1),AN,BN,NABR)
c Graupel
         call ABSUB(F,AVET,R(NG1),AN,BN,NUM_GRP,NAB,NABR,MixingRule)
         call csdrp(F,NUM_GRP,CS(1,NG1),GN(NG1),AN,BN,NABR)
c Cloud Ice Water
	 call cspar(F,AVET,ICESHAPE,RZ(NI1),NUM_ICE,CS(1,NI1),GN(NI1),
     &              DEN(NI1),IO)
c Snow
	call cspar(F,AVET,SNWSHAPE,RZ(NS1),NUM_SNW,CS(1,NS1),GN(NS1),
     &              DEN(NS1),IO)
c
	call PAR(R,RN,RS(I),WS(I),WI(I),ICEMODEL,SNWMODEL,DEN,Pfac)
c
	call TOTW(R,RN,WW1,WW2) 

	 TW1=TW1+WW1*DZ
	 TW2=TW2+WW2*DZ
c        print*,TW1,TW2

c        if(RR(I).lt.-0.9) RR(I)=RR_CAL(R,RN,Pfac)

	 CALL KCOEF(NR,RN,CS,GN,AIRABS,RC,ASYG)

c	print*,RC,ASYG

         TAU(I)=RC(3)*DZ
	 OMEGA=RC(2)/RC(3)
c	 IF(OMEGA*TAU(I).GT.OM_MAX) OM_MAX=OMEGA*TAU(I)
	 OM_MAX=OM_MAX+OMEGA*TAU(I)
  	 IF(OMEGA.lt.REAL_SMALL) THEN
           OMEGA=REAL_SMALL
 	   ASYG=0
 	 ELSE IF(OMEGA.GT.ALMOST1) THEN
 	   OMEGA=ALMOST1
 	 ENDIF
	 CALL HGPF(NL+1,ASYG,AL)
C
C Delta-4 STREAM Adjustment
C BASED ON Liou (1992)
C
C	IF(OMEGA.GT.0.1 .AND. ABS(ASYG).GT.0.1) THEN
 	 OMEGA0=OMEGA
 	 FAC=AL(NL+1)/(2*NL+3)
  	 OMEGA=(1-FAC)*OMEGA0/(1-OMEGA0*FAC)
  	 ASYG=(ASYG-FAC)/(1-FAC)
  	 TAU(I)=(1-OMEGA0*FAC)*TAU(I)
  	 DO LL1=0,NL
  	  AL(LL1)=(AL(LL1)-FAC*(2*LL1+1))/(1-FAC)
  	 END DO
C	ENDIF
c	print*, OMEGA,OMEGA0,FAC
C
	 B0(I)=BWT(F,TK(I))
	 B1(I)=(BWT(F,TK(I+1))-B0(I))/TAU(I)
         OM(I)=OMEGA
	 DO LL1=0,NL
	  AL0(LL1,I)=AL(LL1)
	 END DO
	 CALL STRMKW(NL,OMEGA,EKJ(1,I),GIJ(1,1,I),AL,B0(I),B1(I),PJ(1,I))

        END DO
	
c	print*, (EKJ(1,kkk),EKJ(2,kkk),kkk=1,L),RSUU
c	print*, (B0(kkk),B1(kkk),(PJ(iii,kkk),iii=1,4),kkk=1,L)
c       print*,(((GIJ(iii,jjj,kkk),iii=1,4),jjj=1,2),kkk=1,L)
c	print*,RSU_DIF,(TAU(kkk),OM(kkk),kkk=1,L)

	IF(IHV.NE.2) THEN

 	IF(OM_MAX.GT.OM_SMALL) THEN
	CALL GETTB(L,F,TS,RSU_DIF,SurfModel,RSUU(1),AL0,OM,B0,B1,PJ,
     &            NS,NL,EKJ,GIJ,TAU,TB0,AX,BX,WK,LOUT,U)
c2003.12.3 RSV --> RSU_DIF
 	ELSE
c	print*,OM_MAX
 	CALL TBNOSCT(L,TS,RSU_DIF,SurfModel,RSUU(1),NS,
     &               TAU,TK,TB0,LOUT,U,WK)
 	ENDIF

	IF(U.GT.0) THEN
	 TB(1)=TB0(1)
	ELSE
	 TB(1)=TB0(2)
	END IF

	END IF

c	print*, (EKJ(1,kkk),kkk=1,L),RSUU


	IF(IHV.NE.1) THEN
	
 	IF(OM_MAX.GT.OM_SMALL) THEN
        CALL GETTB(L,F,TS,RSU_DIF,SurfModel,RSUU(2),AL0,OM,B0,B1,PJ,
     &            NS,NL,EKJ,GIJ,TAU,TB0,AX,BX,WK,LOUT,U)
c2003.12.3 RSH --> RSU_DIF
 	ELSE
 	CALL TBNOSCT(L,TS,RSU_DIF,SurfModel,RSUU(2),NS,
     &               TAU,TK,TB0,LOUT,U,WK)
 	ENDIF

        IF(U.GT.0) THEN
         TB(2)=TB0(1)
        ELSE
         TB(2)=TB0(2)
        END IF

	END IF
	
	RETURN
	END 
