#include <stdio.h>
#include <string.h>
#include <stdlib.h>

float es(float);
float sdfromrh(float, float, float);
float rhfromsd(float, float, float);

/* modify air profile   
THE FOLLOWING OPTIONS ARE AVAILABLE:
 YOU MAY ONLY SPECIFY  ONE (1) OPTION BY EACH CALL!!!
1. T ONLY  iopt=1
     dt=<Delta-T>  Delta-T applies to levels specified by -L,  and
                    specific humidity does NOT change, Relative
		    humidity changes
2. T & Water_vapor iopt=2
     dt=<Delta-T>  Delta-T applies to levels specified by -L,  and
		    Relative humidity does NOT change, Specific
	            humidity changes
3. R. humidity only  iopt=3
     dt=<delta-RH>  Delta-Rh applies to levels specified. 
4. Specific Humidity by ratio iopt=4
     dt=<ratio-SH>  Change RH in levels specified, to make SH=Ratio*SHold

l1=<lowest layer> l2=<highest layer>  
       	Layer counted from BOTTOM UP. APPLY to ALL LEVELS IF NOT SPECIFIED.

fi <input file name>
fo <output file name>

*/

/* -i fi -o fo -b l1 -t l2 -opt iopt -d dt
defaults: fi: stdin, fo: stdout, l1:0 l2:0 iopt:0 dt:0
*/

int main (int argc, char **argv) {

	int i, j, k, layer;
	float z, p, rh,t;
	float sd, ratio, dt;
	char wk[80];
	FILE *fp1, *fp2;
	int iopt, l1, l2;

/* defaults */
	fp1 = stdin;
	fp2 = stdout;
	l1 = 0;
	l2 = 0;
	iopt = 0;
	dt = 0.0;

/* usage */
	if ( (argc < 2) || !strcmp(argv[1],"-h") || !strcmp(argv[1],"-help") ) {
	  fprintf(stderr,"Usage: modair2 [-i infile] [-o outfile] [-b bottom] [-t top] [-opt option] [-d delta]\n");
	  fprintf(stderr,"infile: input file name [stdin]\n");
	  fprintf(stderr,"outfile: output file name [stdout]\n");
	  fprintf(stderr,"bottom: lowest layer number to be applied [1]\n");
	  fprintf(stderr,"top: highest layer number to be applied [top layer]\n");
	  fprintf(stderr,"option[0]: 0= do nothing, 1=change T while S. Humidity unchanged\n");
	  fprintf(stderr,"           2=change T while R. Humidity unchanged\n");
	  fprintf(stderr,"           3=change R. Humidity\n");
	  fprintf(stderr,"           4=change S. humdity by ratio\n"); 
	  fprintf(stderr,"delta[0.0]: units: K/%%/ratio\n");
	  exit (0);
	}

/* check cmdline */

	i = 1;
	while (i < argc) {	
	 if (!strcmp(argv[i],"-opt")) {
	 	sscanf(argv[i+1],"%d",&iopt);
		if (iopt>4) {
			fprintf(stderr,"iopt must be 0 - 4, iopt=%d\n",iopt);
			exit (1);
		}
	 }
	 if (!strcmp(argv[i],"-i")) {
		if (!(fp1 = fopen(argv[i+1],"r"))) {
			fprintf(stderr,"FILE %s does not exist.\n",argv[i+1]);
			exit (1);
	 	}
	 }
	 if (!strcmp(argv[i],"-o")) 
		fp2 = fopen(argv[i+1],"w");
	 if (!strcmp(argv[i],"-b"))
		sscanf(argv[i+1],"%d",&l1);
	 if (!strcmp(argv[i],"-t"))
		sscanf(argv[i+1],"%d",&l2);
	 if (!strcmp(argv[i],"-d"))
		sscanf(argv[i+1],"%f",&dt);

	 i += 2;
	}

	if (iopt == 4 && dt <= 0.) dt = 1.;

	fgets(wk,sizeof(wk),fp1);
	fprintf(fp2,"%s",wk);
	sscanf(wk,"%d",&layer);

	j = (l1 == 0) ? 0 : l1-1;
	k = (l2 == 0) ? layer-1 : l2;

	for (i=0;i<layer;i++) {
	  fscanf(fp1,"%f %f %f %f",&z,&p,&t,&rh);
	  switch(iopt) {
	   case 0:
		break;
	   case 1:
            if (i>=j && i<=k) {
	      sd = sdfromrh(p,t,rh);
	      t += dt;
	      rh = rhfromsd(p,t,sd);
	      if(rh>100) rh=100;
	    }
	    break;
	   case 2:
	    if (i>=j && i<=k) 
                	    t += dt;
	    break;
	   case 3:
	    if (i>=j && i<=k)
			    rh += dt;
			    if(rh<0.) rh=0.;
			    if(rh>100) rh=100;
	    break;
	   case 4:
	    if (i>=j && i<=k) {
	      sd = sdfromrh(p,t,rh);
              sd *= dt;
	      if(sd<0.) sd=0.;
	      rh = rhfromsd(p,t,sd);
	      if(rh>100) rh=100;
	    }
	    break; 
	  }
          fprintf(fp2,"%f   %f   %f   %f\n",z,p,t,rh);
	}
	
	fclose(fp1);
	fclose(fp2);
}
 
